@@ -3,11 +3,8 @@
 QueenbeeTask is a wrapper around luigi.Task which is used by queenbee-local to
 execute Queenbee-specific task. You probably don't need to use this module in your code.
 """
-import pathlib
 import logging
 import subprocess
-import shlex
-import re
 import socket
 import os
 import shutil
@@ -47,6 +44,10 @@ class QueenbeeTask(luigi.Task):
         * supports piping
     """
 
+    def get_interface_logger(self):
+        logger = logging.getLogger('luigi-interface')
+        return logger
+
     @property
     def input_artifacts(self):
         """Task's input artifacts.
@@ -88,7 +89,7 @@ class QueenbeeTask(luigi.Task):
         Args:
             dst: Execution folder.
         """
-        logger = logging.getLogger('luigi-interface')
+        logger = self.get_interface_logger()
         for art in self.input_artifacts:
             logger.info(f"copying input artifact: {art['name']}...")
             try:
@@ -107,7 +108,7 @@ class QueenbeeTask(luigi.Task):
         Args:
             src: Execution folder.
         """
-        logger = logging.getLogger('luigi-interface')
+        logger = self.get_interface_logger()
         for art in self.output_artifacts:
             logger.info(f"copying output artifact: {art['name']}...")
             _copy_artifacts(os.path.join(src, art['from']), art['to'])
@@ -118,112 +119,11 @@ class QueenbeeTask(luigi.Task):
         Args:
             src: Execution folder.
         """
-        logger = logging.getLogger('luigi-interface')
+        logger = self.get_interface_logger()
         for art in self.output_parameters:
             logger.info(f"copying output parameters: {art['name']}...")
             _copy_artifacts(os.path.join(src, art['from']), art['to'])
 
-    @staticmethod
-    def _pre_process_command(input_command):
-        """Process input command before execution.
-
-        This method:
-            1. separates output file if the output is redirected to a file using >
-            2. breaks-down the command into several commands if piping is happening |
-        """
-        def _raw(text):
-            """Raw string representation.
-
-            This is useful to handle edge cases in Windows.
-            """
-
-            escape_dict = {
-                '\a': r'\a', '\b': r'\b', '\c': r'\c', '\f': r'\f', '\n': r'\n',
-                '\r': r'\r', '\t': r'\t', '\v': r'\v', '\'': r'\'', '\"': r'\"',
-                '\0': r'\0', '\1': r'\1', '\2': r'\2', '\3': r'\3', '\4': r'\4',
-                '\5': r'\5', '\6': r'\6', '\7': r'\7', '\8': r'\8', '\9': r'\9'
-            }
-
-            new_string = ''
-            for char in text:
-                try:
-                    new_string += escape_dict[char]
-                except KeyError:
-                    new_string += char
-            return new_string
-
-        output_cmd = []
-        input_command = pathlib.Path(input_command).as_posix()
-        commands = ' '.join(input_command.split()).split('|')
-        STDOUT = (-1, '')  # -1 is PIPE
-
-        for count, command in enumerate(commands[:-1]):
-
-            assert '>' not in command.split('&&')[-1], \
-                'You cannot redirect stdout with > and pipe the' \
-                ' outputs at the same time:\n\t%s' % command
-
-            # check for stdin
-            # -1 means use of stdout from the command before this command
-            stdin = -1 if count != 0 else ''
-            if '<' in command:
-                if count == 0:
-                    stdin = _raw(shlex.split(command.split('<')[-1])[0])
-                    # replace stdin from the original command
-                    command = re.sub(r"\s<+\s\S+", '', command)
-                else:
-                    raise ValueError(
-                        'You cannot use < for stdin and pipe data to command '
-                        'at the same time:\n\t%s' % command
-                    )
-
-            output_cmd.append(
-                {
-                    'cmd': command.strip(),
-                    'stdin': stdin.strip() if stdin != -1 else stdin,
-                    'stdout': STDOUT
-                }
-            )
-
-        # set-up stdout for the last command
-        last_command = commands[-1]
-        mode = None
-        if '>>' in last_command:
-            mode = 'a'
-            stdout = _raw(shlex.split(last_command.split('>>')[-1])[0])
-        elif '>' in last_command:
-            mode = 'w'
-            stdout = _raw(shlex.split(last_command.split('>')[-1])[0])
-        else:
-            stdout = subprocess.PIPE
-
-        command = re.sub(r"\s>+\s\S+", '', last_command)
-
-        stdin = None if len(commands) == 1 else -1
-        if '<' in command:
-            # now check for stdin
-            stdin = _raw(shlex.split(command.split('<')[-1])[0])
-            # replace stdin from the original command
-            command = re.sub(r"\s<+\s\S+", '', command)
-
-        output_cmd.append(
-            {
-                'cmd': command.strip(),
-                'stdin': stdin,
-                'stdout': (stdout, mode)
-            }
-        )
-        return output_cmd
-
-    def _stream_file_content(self, file_object):
-        """Return contents of a file like object as a single string."""
-        try:
-            file_object.seek(0)
-        except AttributeError:
-            return str(file_object)
-        else:
-            return ''.join(map(lambda s: s.decode('utf-8'), file_object.readlines())).split('\n')
-
     @property
     def _is_debug_mode(self):
         if '__debug__' not in self._input_params:
@@ -256,8 +156,9 @@ class QueenbeeTask(luigi.Task):
         return dst_dir, dst
 
     def run(self):
-        logger = logging.getLogger('luigi-interface')
-
+        logger = self.get_interface_logger()
+        # this is a temporary solution until we fix the bug with logging on Windows
+        logs = []
         # replace ' with " for Windows systems and vise versa for unix systems
         command = self.command()
         if SYSTEM == 'Windows':
@@ -270,28 +171,26 @@ class QueenbeeTask(luigi.Task):
         os.chdir(dst)
 
         self._copy_input_artifacts(dst)
+        logger.info(f'Running {self.__class__.__name__}...')
 
         p = subprocess.Popen(
             command, stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE, shell=True, env=os.environ
+            stderr=subprocess.STDOUT, shell=True, env=os.environ
         )
 
-        stdout, stderr = p.communicate()
+        for line in iter(p.stdout.readline, b''):
+            msg = line.decode('utf-8').strip()
+            logs.append(msg)
+            logger.info(msg)
 
-        if p.returncode != 0 and stderr != b'':
-            errors = self._stream_file_content(stderr)
-            logger.error(errors)
-            raise RuntimeError(errors)
+        p.communicate()
 
-        if stderr is not None:
-            # this is a rare case but happens with Radiance commands where it print
-            # to stderr for verbose reporting
-            out_info = self._stream_file_content(stderr)
-            logger.info(out_info)
-
-        if stdout is not None:
-            stdout = self._stream_file_content(stdout)
-            logger.info(stdout)
+        if p.returncode != 0:
+            logger.error(
+                f'Failed task: {self.__class__.__name__}. '
+                'Run the command with --debug key to debug the workflow.'
+            )
+            raise ValueError('\n' + '_' * 20 + '\n' + '\n'.join(logs))
 
         # copy the results file back
         self._copy_output_artifacts(dst)
